import logging
import os
from typing import Tuple
import boto3
from botocore.client import Config

# from osdu_api.providers.blob_storage import get_client
from osdu_api.providers.types import FileLikeObject

from bundles.storage import BundleStorageClient

logger = logging.getLogger(__name__)


class FileNotFound(Exception):
    def __init__(self, filename: str):
        self.message = f"ERROR: File {filename} was not found."
        super().__init__(self.message)


class IBMBundleStorageClient(BundleStorageClient):
    def __init__(self) -> None:
        # self.client = get_client()
        self.bucket_name = os.environ["POLICY_BUCKET"]
        self.content_type = "application/x-gtar"
        self.s3_client = boto3.client(
            "s3",
            endpoint_url=os.environ["ENDPOINT_URL"],
            aws_access_key_id=os.environ["AWS_ACCESS_KEY_ID"],
            aws_secret_access_key=os.environ["AWS_SECRET_ACCESS_KEY"],
            config=Config(signature_version="s3v4"),
            region_name="us-east-1",
            verify=os.environ["ssl_verify"],
        )
        logger.debug(f"IBM storage init {self.bucket_name}")

    def download_file(
        self, key: str, file: FileLikeObject
    ) -> Tuple[FileLikeObject, str]:
        logger.debug(f"IBM storage download bucket: {self.bucket_name} file:{file}")
        try:
            self.s3_client.download_fileobj(self.bucket_name, key, file)
            return file, key
        except Exception as e:
            logger.error(f"Failed to download file {e} from {self.bucket_name}")

    def upload_file(self, key: str, file: FileLikeObject) -> str:
        try:
            self.s3_client.upload_fileobj(file, self.bucket_name, key)
            return file, key
        except Exception as e:
            logger.error(f"Failed to upload file {e} to {self.bucket_name}")
